function [K, Q_u, lambda] = getStiffnessMatrix( FEM, SD )
%%  Initialize variables
    %Coefficient matrices
    E0 = zeros(SD.nDOF, SD.nDOF);
    E1 = zeros(SD.nDOF, SD.nDOF);
    E2 = zeros(SD.nDOF, SD.nDOF);
    
%%  Numerical integration values
    [ samplingPoints , weights ] = integrationParameters1D ( FEM.nodesPerElement , FEM.quadratureType);
    numberOfSamplingPoints = length(samplingPoints);
    
%%  Coefficient matrices assembly loop 
    %loop over all elements --> fully assembled EX_subdomain
    for i = 1:1:SD.numberOfElements

        DOFPerElement = FEM.DOFPerNode * FEM.nodesPerElement;
        E0_temp = zeros( DOFPerElement, DOFPerElement );
        E1_temp = zeros( DOFPerElement, DOFPerElement );
        E2_temp = zeros( DOFPerElement, DOFPerElement );

        %recall (x,y)-coordinates of each elements nodes
        [ xCoord , yCoord ] = elementNodeCoordinates( SD.nodes , SD.localElements , i , FEM.nodesPerElement, SD.SC);

        %recall material properties for each element to form corresponding matrix
        [ D ] = constitutiveMatrix2D( SD.E , SD.v , FEM.constitutiveLaw );

        %loop over the integration points --> fully integrated coefficient matrices EX_temp
        for j = 1:1:numberOfSamplingPoints
            %recall specific shape functions and their derivatives
            samplingPoint = samplingPoints(j);
            [shapeFunction , shapeFunctionDerivative] = shapeFunction1D (FEM.nodesPerElement , samplingPoint, FEM.quadratureType);

            numberOfShapeFunctions = length(shapeFunction);

            %recall specific integration weights
            weight = weights(j);

            %recall interpolation of cartesian coordinates (x,y) and their derivates
            x   = shapeFunction * xCoord;
            y   = shapeFunction * yCoord;
            x_n = shapeFunctionDerivative * xCoord;
            y_n = shapeFunctionDerivative * yCoord;

            %calculate specific jacobian and it's determinant
            detJ = x * y_n - y * x_n;
            %specify shape function for each DOF
                %initialize shape function and it's derivatives in 2D
                    shapeFunction2D           = zeros( FEM.DOFPerNode , DOFPerElement );
                    shapeFunctionDerivative2D = zeros( FEM.DOFPerNode , DOFPerElement );

                %construct shape functions and associated derivaties in 2D
                    positionCounter = 1;
                    for k = 1:1:numberOfShapeFunctions
                        %abbreviation for simple notation
                        positionRow = 1:FEM.DOFPerNode;
                        positionCol = positionCounter : positionCounter + FEM.DOFPerNode - 1;

                        shapeFunction2D(positionRow,positionCol)           = shapeFunction(k)           * eye(2);
                        shapeFunctionDerivative2D(positionRow,positionCol) = shapeFunctionDerivative(k) * eye(2);
                        positionCounter = positionCounter + FEM.DOFPerNode;
                    end

            %calculate strain desplacement matrices B1 B2
            B1 =   1/detJ * [y_n ,   0 ;...
                               0 , -x_n;...
                            -x_n ,  y_n] * shapeFunction2D;
            B2 =   1/detJ * [ -y ,   0 ;...
                               0 ,   x ;...
                               x ,  -y ] * shapeFunctionDerivative2D;

            %find specific contributrion to coefficient matrices
            E0_temp = E0_temp + transpose(B1) * D * B1 * detJ * weight;
            E1_temp = E1_temp + transpose(B2) * D * B1 * detJ * weight;
            E2_temp = E2_temp + transpose(B2) * D * B2 * detJ * weight;

        end
        
        %save coefficient matrix contribution in correct position
            %determine saving position
                ind = nodesToDOF(SD.localElements(i,:)',FEM.DOFPerNode);

            %implement proper storing procedure
                E0(ind, ind) = E0(ind, ind) + E0_temp(:,:);
                E1(ind, ind) = E1(ind, ind) + E1_temp(:,:);
                E2(ind, ind) = E2(ind, ind) + E2_temp(:,:);
    end

%%  Construct Hamiltonian
    %form Hamiltonian matrix Z
    invE_0 = E0\eye(size(E0));

    Z11 = invE_0 * E1';
    Z12 = - invE_0;
    Z21 = - E2 + (E1 * invE_0 * E1');
    Z22 = - E1 * invE_0;

    Z = [Z11, Z12; Z21, Z22];
    
    %extract parameters
    [rowZ, ~ ] = size(Z);
    
%%  Perform Schur decomposition (UNSW)
    [ v, s, ~ ] = BlockDiagSchurForm( Z, 2, 0.2, FEM.tol );
    lambda = diag(s); lambda = lambda(1:rowZ/2);
    K = v(rowZ/2+1:end,1:rowZ/2)/v(1:rowZ/2,1:rowZ/2);
    K = (1/2)*(K + K');
    
    Q_u = v(1:rowZ/2,1:rowZ/2);
end