%%  Global Input Parameters
    [FEM,SD] = inputFile_4();
    
%%  Subdomains
    %Parameters
    FEM.nSD = length(SD);
    
%%  Generate Mesh
    if FEM.verbose; fprintf(1,' 3. Meshing subdomain(s)'); end
    %generate subdomain meshes with nodes, element connectivity and information on element distribution
    for i = 1:1:FEM.nSD
        if FEM.verbose; fprintf(' %d',i); end
        if FEM.quadratureType == 1
            [SD(i).nodes, SD(i).localElements, SD(i).elementsPerSide] = generateMesh_equalSpacing(FEM,SD(i));
        elseif FEM.quadratureType == 2
            [SD(i).nodes, SD(i).localElements, SD(i).elementsPerSide] = generateMesh_LobattoSpacing(FEM,SD(i));
        end
        [ SD(i).numberOfNodes , ~ ]    = size(SD(i).nodes);
        [ SD(i).numberOfElements , ~ ] = size(SD(i).localElements);
        SD(i).nDOF = FEM.DOFPerNode * SD(i).numberOfNodes;
        SD(i).sumEPS = cumsum(SD(i).elementsPerSide);
        if FEM.verbose; fprintf('\b\b'); end
    end
    
    if FEM.verbose; fprintf(1,'...done!\n'); end
    
%%  Plot mesh
    if FEM.plotMesh == 1; plotSBFEM_Mesh; end
   
%%  Global nodes 
    if FEM.verbose; fprintf(1,' 4. Generating global nodes...'); end
    FEM.nodes = [];
    for i = 1:1:FEM.nSD
        FEM.nodes = [FEM.nodes;SD(i).nodes];
    end
   
    [FEM.nodes, indOld, indNew] = uniquetol(FEM.nodes,eps,'ByRows',true);

    for i = 1:1:FEM.nSD
        startInd = sum([SD(1:i).numberOfNodes])-SD(i).numberOfNodes+1;
        endInd   = startInd + SD(i).numberOfNodes-1;
        SD(i).gNodeIndex = [startInd,endInd];
        SD(i).gNodes = indNew(SD(i).gNodeIndex(1):SD(i).gNodeIndex(2));
        SD(i).gDOF = nodesToDOF(SD(i).gNodes,FEM.DOFPerNode);
    end

    FEM.nDOF = size(FEM.nodes,1)*FEM.DOFPerNode;
    
    if FEM.verbose; fprintf('done!\n'); end
    
%%  Compute Stiffness matrix and assemble
    if FEM.verbose; fprintf(1,' 5. Generating and assembling K'); end
    %initialize
    FEM.K = zeros(FEM.nDOF);
    for i = 1:1:FEM.nSD
        if FEM.verbose; fprintf(' (%d/%d)',i,FEM.nSD); end
        %calculate stiffness matrix
        [SD(i).K,SD(i).Q,SD(i).lambda] = getStiffnessMatrix(FEM,SD(i));
        %assemble
        FEM.K(SD(i).gDOF,SD(i).gDOF) = FEM.K(SD(i).gDOF,SD(i).gDOF) + SD(i).K;
        if FEM.verbose; fprintf('\b\b\b\b\b\b'); end
    end
    
    if FEM.verbose; fprintf(1,'...done!\n'); end
    
%%  Compute Loading Vector and assemble
    if FEM.verbose; fprintf(1,' 6. Generating and assembling F'); end
    %initialize
    FEM.F = zeros(FEM.nDOF,1);
    for i = 1:1:FEM.nSD   
        if FEM.verbose; fprintf(' (%d/%d)',i,FEM.nSD); end
        
        %calculate force vector
        [SD(i).F] = getForceVectorTension(FEM,SD(i));
        %assemble
        FEM.F(SD(i).gDOF,1) = FEM.F(SD(i).gDOF,1) + SD(i).F;
        
        if FEM.verbose; fprintf('\b\b\b\b\b\b'); end
    end
    
    if FEM.verbose; fprintf(1,'...done!\n'); end

%%  Apply Boundary Conditions
    if FEM.verbose; fprintf(1,' 7. Applying boundary conditions'); end
    
    [FEM.BC] = getBoundaryConditions(FEM,SD);
    
    %apply BC
    FEM.K(FEM.BC,     :) = zeros;
    FEM.K(     :,FEM.BC) = zeros;
    FEM.K(FEM.BC,FEM.BC) = eye(length(FEM.BC));
    FEM.F(FEM.BC,1)      = zeros;
    
    if FEM.verbose; fprintf(1,'...done!\n'); end

%%  Compute Displacements
    if FEM.verbose; fprintf(1,' 8. Solving U = K\\F'); end
    FEM.U = FEM.K\FEM.F;
    if FEM.verbose; fprintf(1,'...done!\n'); end
    
    for i =  1:1:FEM.nSD
        SD(i).U = FEM.U(SD(i).gDOF,1);
    end
 
%%  Calculate Constants
    if FEM.verbose; fprintf(1,' 9. Solving for C'); end
    
    for i =  1:1:FEM.nSD
        if FEM.verbose; fprintf(' (%d/%d)',i,FEM.nSD); end
        SD(i).C = SD(i).Q \ SD(i).U;
        if FEM.verbose; fprintf('\b\b\b\b\b\b'); end
    end
    
    if FEM.verbose; fprintf(1,'...done!\n'); end

%%  Plot displacements
    if FEM.plotDisplacements == 1; plotSBFEM_Displacements; end
    
%%  Calculate SIFs
    if FEM.verbose; fprintf(1,'10. Calculating SIFs'); end
    for i = 1:1:FEM.nSD
        if SD(i).cracked
        %general quantities
            %find crack line extension and associated quatities
                [SD(i).patchElements, SD(i).eta, SD(i).dist, SD(i).smoothingFlag] = findPatchElements(FEM,SD(i));

            %find quantities contributing to singular response
                [SD(i).singularInd, SD(i).singularLambda, SD(i).singularC] = findSingularResponse(FEM, SD(i));
                
        %calculate SIFs based on raw stresses
            [SD(i).SIFs_raw] = computeKRaw(FEM,SD(i));
            
        %calculate SIFs based on recovered stresses
            %[SD(i).SIFs_rec] = computeKRec(FEM,SD(i));
        
        %error indicator
            %SD(i).SIFs_err = abs(SD(i).SIFs_rec-SIFs_raw);
        end
    end
    
    if FEM.verbose; fprintf(1,'...done!\n'); end
    
%%  Timings
    if FEM.verbose; fprintf('******************************************\n'); end
    if FEM.timing
        FEM.analysisTime = toc(FEM.tStart);
        if FEM.plotMesh || FEM.plotDisplacements
            fprintf(1,'SBFEM analysis WITH PLOTS completed in %.4f seconds\n',FEM.analysisTime)
        else
            fprintf(1,'SBFEM analysis completed in %.4f seconds\n',FEM.analysisTime)
        end
    end

%%  Testing
    
%%  Output
    
%%  Clean up
    clear endInd startInd indNew indOld i j xFill yFill
    


    

    
    
    

    
    
    